function HEAT_WAVE_1D(N, eqn, boundary_condition)

if( nargin < 1 )
   N = 10;
end

is_heat = false;
if( nargin < 2 || (nargin>=2 && strcmp(eqn, 'HEAT')) )
   is_heat = true;
end

do_dirichlet = true;
if( nargin >= 3 && ~strcmp(boundary_condition, 'dirichlet') )
   do_dirichlet = false;
end

a = 0;
b = 100;

[grid, A] = discretize_laplace_1d(a, b, N, do_dirichlet);

if( is_heat )
   rhs_func = @(x, middle) (sign(middle-10-x)*sign(x-middle-10))+1;

   u0 = @(x) (sign(40-x)*sign(x-60))+1;
   y0 = 20*discretize_function_1d(grid, u0);

   MAX_TEMP = 50;
   MIN_TEMP = 0;
else
   rhs_func = @(x, middle) exp(-(x-middle)^2/15);

   u0 = @(x) 0;
   y0 = discretize_function_1d(grid, u0);

   y1 = y0;

   MAX_TEMP =  1200;
   MIN_TEMP = -1200;
end

% initialize callbacks
clf;
fig_handle = gcf;

ah = axes('XLim', [a, b], 'YLim', [MIN_TEMP, MAX_TEMP]);

plot([a, b], [MAX_TEMP, MIN_TEMP], 'kx', 'MarkerSize', 0.001);
hold on;

set(fig_handle, 'WindowButtonDownFcn', @mouse_down);
set(fig_handle, 'WindowButtonUpFcn', @mouse_up);
set(fig_handle, 'WindowButtonMotionFcn', @mouse_move);
set(fig_handle, 'KeyPressFcn',@keyPressCallback);

plot_handle = plot(grid, y0, 'b-', 'YDataSource', 'act_y');
hold off;

title('press any keyboard key to quit');

rhs_fact = 0.0;
rhs_middle = 0.0;

terminate = false;

if( is_heat )
   hT = 2;

   while( ~terminate )
      if( rhs_fact~=0.0 )
         f0 = rhs_fact/10*discretize_function_1d(grid, @(x) rhs_func(x, rhs_middle));
      else
         f0 = zeros(size(y0));
      end

      % HEAT
%      curr_fun = @(t, y) A*y + f0;
%      [times, sols] = ode45(curr_fun, [0, hT], y0);
%      y0 = sols(end, :)';

      y0 = y0 + hT*A*y0 + f0;

      act_y = y0;

      % plot actual state
      refreshdata(plot_handle, 'caller');
      drawnow;
   end
else
   hT = 0.5;

   while( ~terminate )

      if( rhs_fact~=0.0 )
         f0 = rhs_fact*discretize_function_1d(grid, @(x) rhs_func(x, rhs_middle));
      else
         f0 = zeros(size(y0));
      end

      % WAVE
      act_y = 2*y1 - y0 + hT^2*(A*y1 + 5*f0);
      y0 = y1;
      y1 = act_y;

      % plot actual state
      refreshdata(plot_handle, 'caller');
      drawnow;
   end

   y1 = y0;
end


close(fig_handle);

   %
   % Callback functions
   %
   function mouse_down(src, data)
      pnt = get(ah, 'CurrentPoint');
      x0 = pnt(1,1);
      u0 = pnt(1,2);

      rhs_middle = x0;

      switch( get(fig_handle, 'SelectionType') )
         case 'normal' % left mouse button
            rhs_fact = 1.0;
         case 'alt' % left mouse button
            rhs_fact = -1.0;
         case 'extend' % left mouse button
            terminate = true;
      end
   end

   function mouse_up(src, data)
      pnt = get(ah, 'CurrentPoint');
      x0 = pnt(1,1);
      u0 = pnt(1,2);

      rhs_fact = 0.0;
   end

   function mouse_move(src, data)
      pnt = get(ah, 'CurrentPoint');
      x0 = pnt(1,1);
      u0 = pnt(1,2);

      rhs_middle = x0;
   end

   function keyPressCallback(src, data)
      terminate = true;
   end
end
