function RUN_HEAT(grid, T, h)

if( nargin == 0 )
   load('grid_cross.mat');
   T = 0.25;
   h = 0.001;
elseif( nargin~= 3 )
   error('You have to specify 0 or 3 parameters!');
end

A = discretize_laplace(grid);

n = size(A, 1);

y = zeros(n, 1);

Z = vec_to_f2d(grid, y, 5);
hand = surf(grid.grid_X, grid.grid_Y, Z)
set(hand, 'ZDataSource', 'Z');

campos( [   -12.1188   -1.3808  122.3448 ]);

E = inf;

func_f = @(t) f2d_to_vec(grid, eval_rhs_heat(grid.grid_X, grid.grid_Y, t));

ab_func = @(t, y) A*y + func_f(t);

opti = odeset('RelTol', 1e-2, 'AbsTol', 1e-2);

times = 0:h:T;
[times, sol] = ode45(ab_func, times, zeros(n, 1), opti);
sol = sol';


while(true)

   for ind = 1:length(times)
      Z = vec_to_f2d(grid, sol(:, ind), 7);

      title(['Time : ',num2str(times(ind)), '; Energy : ', num2str(E)]);

      refreshdata(hand, 'caller');
      drawnow;
      pause(h);
   end

   waitforbuttonpress;
end

function Z = vec_to_f2d(grid, y, max_val);

Z = zeros(size(grid.grid_X));

for i = 1:size(Z, 1)
   for j = 1:size(Z, 2)
      id = grid.grid_ids(i,j);
      if( id>0 )
         Z(i,j) = y(id);
      else
         Z(i,j) = 0;
      end
   end
end

if( nargin > 2 )
   Z(end,1) = max_val;
   Z(end,end) = -max_val;

   Z = max(min(Z, max_val), -max_val);
end


function x = f2d_to_vec(grid, Z);

x = zeros(grid.np, 1);

for i = 1:size(Z, 1)
   for j = 1:size(Z, 2)
      id = grid.grid_ids(i,j);
      if( id>0 )
         x(id) = Z(i,j);
      end
   end
end
