function RUN_HEAT(grid, T, h)

if( nargin == 0 )
   load('pipe_domain.mat');
   grid = domain_to_grid(domain_pipe,0.1);
   T = 3;
   h = 0.001;
elseif( nargin~= 3 )
   error('You have to specify 0 or 3 parameters!');
end

A = discretize_laplace(grid);

n = size(A, 1);

y = zeros(n, 1);
v = zeros(n, 1);

Z = vec_to_f2d(grid, y, 5);
hand = surf(grid.grid_X, grid.grid_Y, Z)
set(hand, 'ZDataSource', 'Z');

E = inf;

c= 2;

disp('building matrix');
bigA = spalloc(2*n, 2*n, nnz(A)+n);
bigA(n+1:2*n, 1:n) = c^2*A;
bigA(1:n, n+1:2*n) = speye(n);

%max(abs(real(eig(full(bigA)))))


disp('factoring matrix');
[L, U, p] = lu(speye(2*n)-h*bigA, 'vector');

disp('matrix factored');

count = 0

for t=0:h:T
   count = count + 1;
   g = eval_rhs_wave(grid.grid_X, grid.grid_Y, t);
   %f = [zeros(n, 1); f2d_to_vec(grid, g)];
   f = f2d_to_vec(grid, g);
   y(f~=0) = f(f~=0);


   if( mod(count, 2) == 0 )
      %all_vec = [y; v] + h*(bigA*[y; v] + f);
      all_vec = [y; v] + h*(bigA*[y; v]);
   else
   %b = [y; v] + h*f;
      b = [y; v];
      all_vec = U\(L\b(p));
   end
   y = all_vec(1:n);
%
   v = all_vec(n+1:2*n);




%   newy = 2*y - v + h^2 * A * y;
%   v = y;
%   y = newy;


   Z = vec_to_f2d(grid, y, 1.5);

   title(['Time : ',num2str(t), '; Energy : ', num2str(E)]);

   refreshdata(hand, 'caller');
   drawnow;
   pause(h);
end


function Z = vec_to_f2d(grid, y, max_val);

Z = zeros(size(grid.grid_X));

for i = 1:size(Z, 1)
   for j = 1:size(Z, 2)
      id = grid.grid_ids(i,j);
      if( id>0 )
         Z(i,j) = y(id);
      else
         Z(i,j) = 0;
      end
   end
end

if( nargin > 2 )
   Z(end,1) = max_val;
   Z(end,end) = -max_val;

   Z = max(min(Z, max_val), -max_val);
end


function x = f2d_to_vec(grid, Z);

x = zeros(grid.np, 1);

for i = 1:size(Z, 1)
   for j = 1:size(Z, 2)
      id = grid.grid_ids(i,j);
      if( id>0 )
         x(id) = Z(i,j);
      end
   end
end
